#!/opt/bin/perl5 -w

#=====================================================================
#
# sortdiff.pl
#
#=====================================================================
#
# Quick hack that takes two files or a series of 
# common files from two given directories, sorts each 
# file then diffs the pair. 
#
#      NOT ANYWHERE NEAR A FULL 'diff' WORKALIKE 
#
# Useful for comparing two files when output should 
# be same in content, but not necessarily in order.
#
# 
# Brian O'Mara - 02/14/2000
#

use strict;
use Getopt::Std;
my $progName = "sortdiff.pl";
my $debug = 0;

my ($source, $test) = &ProcessCmdLine();
if (!-e $source) {print "$progName: $source: No such file or directory\n"; exit;}
if (!-e $test  ) {print "$progName: $test: No such file or directory\n"  ; exit;}

if ((-f $source) && (-f $test)) {
  &CheckFiles($source, $test);
}
elsif ((-d $source) && (-d $test)) {
  opendir(SOURCE, $source)   or die "$progName: Can't open source $source, $!";
  opendir(TEST,   $test)     or die "$progName: Can't open test $test, $!";
  
  my @sourcefiles = grep !/^\./, readdir SOURCE; # grep skips .* files
  my @testfiles   = grep !/^\./, readdir TEST; 
  my %allfiles;
  
  # grab a union of all files in both dirs to loop over.
  foreach my $file (@sourcefiles) { $allfiles{$file}++; }
  foreach my $file (@testfiles)   { $allfiles{$file}++; }
  
  foreach my $filename (sort keys %allfiles) {
    if (!(-e "$source/$filename") || !(-e "$test/$filename")) {
      print STDERR "$progName: Skipping $filename :\n\t*-- Doesn't exist in both $source and $test\n";
    }
    else {
      if ((-z "$source/$filename") || (-z "$test/$filename")) {
	print STDERR "$progName: Skipping $filename :\n\t*-- At least one of $source/$filename or $test/$filename has a file size of 0\n";
      }
      else {
	&CheckFiles("$source/$filename","$test/$filename");
      }
    }
  }
}
else {
  die "$progName: arguments must either both filenames or both dirnames\n";
}

sub CheckFiles() {
  my ($file1, $file2) = @_;
  
#  print "Checking $file1 \tvs. $file2\n";
  print "\n\n";
  print "---------------------------------------------------------------------------\n";
  print "SRC: $file1  ---  TST: $file2\n";
  print "---------------------------------------------------------------------------\n";
  
  `sort $file1 > $file1.sorted1`;
  `sort $file2 > $file2.sorted2`;
  my @output = `diff -wi $file1.sorted1 $file2.sorted2`;
  foreach my $line (@output) {
    if ($debug) {
      $line =~ s/^>/TST\:/;
      $line =~ s/^</SRC\:/;
      print $line;
    }
    else {
      next if ($line =~ /\;/);         # Not interested in comments
      next if ($line =~ /\s+SOA\s+/);  # or SOA format differences
#      next if ($line =~ /\s+HINFO\s+/);  # or HINFO for now
#      next if ($line =~ /\s+TXT\s+/);  # or TXT for now
      next if ($line =~ /^>\s*$/);     # or blank lines
      next if ($line =~ /^<\s*$/);
      
      if (($line =~ /^>/) || ($line =~ /^</)) { 
	$line =~ s/^>/TST\:/;
	$line =~ s/^</SRC\:/;
	print $line; 
      } 
    }
  }
  `rm $file1.sorted1`;
  `rm $file2.sorted2`;
}

#---------------------------------------------------------------------
#                                                       ProcessCmdLine
#
# Get command line options
#
#---------------------------------------------------------------------

sub ProcessCmdLine() {
  if (@ARGV != 2) {
    print "$progName: two filename or dirname arguments required\n";
    exit;
  }
  else {
    my $source       = shift @ARGV;
    my $test         = shift @ARGV;
    return ($source, $test);
  }                                       
}



